package MT::Plugin::Jaccessibility;
use strict;

require MT::Plugin;

@MT::Plugin::Jaccessibility::ISA = qw(MT::Plugin);

our $version = "0.2";

my $plugin = new MT::Plugin::Jaccessibility({
	name => 'Jaccessibility',
	version => $version,
	description => 'ブログのアクセシビリティ向上を支援します',
	author_name => 'Alfasado,Inc.',
	author_link => 'http://alfasado.net/',
	settings => new MT::PluginSettings([
	]),
});

MT->add_plugin($plugin);

MT::Template::Context->add_global_filter(jaccessibility => \&_jaccessibility);

sub _jaccessibility {
	my($text, $arg, $ctx) = @_;
	my @plug_args;
	if($arg == 0){
		@plug_args = (1, 2, 3, 4);
	}else{
		@plug_args = split(/\,/,$arg);
	}
	use utf8;
	use Encode;
	use Encode::Guess qw/ utf8 euc-jp shiftjis 7bit-jis /;
	binmode STDOUT,":utf8";
	$text = decode('utf-8', $text);
	if (grep(/1/,@plug_args)) {
		use Unicode::Normalize;
		$text = NFKC($text);
	}
	if (grep(/2/,@plug_args)) {
		$text =~ s/(\p{Ja})\s*/$1/g;
	}
	if (grep(/3/,@plug_args)) {
		$text =~ s/\¥([0-9|\,]{1,})/$1円/g;
		$text =~ s/\\([0-9|\,]{1,})/$1円/g;
		$text =~ s/\&yen\;([0-9|\,]{1,})/$1円/g;
		$text =~ s/\$([0-9|\,]{1,})/$1ドル/g;
		$text =~ s/\&euro\;([0-9|\,]{1,})/$1ユーロ/g;
		$text =~ s/€([0-9|\,]{1,})/$1ユーロ/g;
		$text =~ s/\&cent\;([0-9|\,]{1,})/$1セント/g;
		$text =~ s/¢([0-9|\,]{1,})/$1セント/g;
		$text =~ s/\&pound\;([0-9|\,]{1,})/$1ポンド/g;
		$text =~ s/£([0-9|\,]{1,})/$1セント/g;
	}
	if (grep(/4/,@plug_args)) {
		$text =~ s/H([0-9]{2})[-\/\.]([0-9]{1,2})[-\/\.]([0-9]{1,2})/平成$1年$2月$3日/g;
		$text =~ s/H([0-9]{2})\.(\s[0-9]{1,2})\.(\s[0-9]{1,2})/平成$1年$2月$3日/g;
		$text =~ s/S([0-9]{2})[-\/\.]([0-9]{1,2})[-\/\.]([0-9]{1,2})/昭和$1年$2月$3日/g;
		$text =~ s/S([0-9]{2})\.(\s[0-9]{1,2})\.(\s[0-9]{1,2})/昭和$1年$2月$3日/g;
		my $S = '-';
		$text =~ s/([0-9]{4})-([0-9]{1,2})-([0-9]{1,2})/&checkdate($1,$2,$3,$S)/eg;
		$S = '/';
		$text =~ s/([0-9]{4})\/([0-9]{1,2})\/([0-9]{1,2})/&checkdate($1,$2,$3,$S)/eg;
		$S = '.';
		$text =~ s/([0-9]{4})\.([0-9]{1,2})\.([0-9]{1,2})/&checkdate($1,$2,$3,$S)/eg;
		$text =~ s/([0-9]{4}):([0-9]{1,2}):([0-9]{1,2})\s\+[0-9]{4}/&checktime($1,$2,$3,':')/g;
		$text =~ s/([0-9]{4}):([0-9]{1,2}):([0-9]{1,2})/&checktime($1,$2,$3,':')/g;
		$text =~ s/([A-Za-z]{3}),\s([0-9]{2})\s([A-Za-z]{3})\s([0-9]{4})/monthToInt($4,$3,$2,$1,"年","月","日")/eg;
	}
	$text = encode('utf-8', $text);
	return $text;
}


sub checktime {
	my($hor, $min, $sec, $sep) = @_;
	if (&time_exists($hor, $min, $sec)) {
		return $hor.'年'.$min.'月'.$sec.'日';
	} else {
		return $hor.$sep.$min.$sep.$sec;
	}
}

sub time_exists {
	my($hor, $min, $sec) = @_;
	if (($hor < 25) && ($min < 61) && ($sec < 61)) {
		return 1;
	} else {
		return 0;
	}
}

sub checkdate {
	my($yer, $mon, $day, $sep) = @_;
	if (&day_exists($yer, $mon, $day)) {
		return $yer.'年'.$mon.'月'.$day.'日';
	} else {
		return $yer.$sep.$mon.$sep.$day;
	}
}

sub day_exists {
	my($year, $month, $day) = @_;
	my(@mlast) = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
	if ($month < 1 || 12 < $month) {
		return 0;
	}
	if ($month == 2) {
		if ((($year % 4 == 0) && ($year % 100 != 0)) || ($year % 400 == 0)) {
			$mlast[1]++;
		}
	}
	if ($day < 1 || $mlast[$month-1] < $day) {
		return 0;
	}
	return 1;
}

sub Ja {
    return <<'END';
+utf8::InHiragana
+utf8::InKatakana
+utf8::Han
END
}

sub monthToInt {
	my($strYear,$strMonth,$strDay,$week,$yStr,$mStr,$dStr)=@_;
	if ($strMonth eq "Jan") {return "$strYear$yStr"."1$mStr$strDay".$dStr};
	if ($strMonth eq "Feb") {return "$strYear$yStr"."2$mStr$strDay".$dStr};
	if ($strMonth eq "Mar") {return "$strYear$yStr"."3$mStr$strDay".$dStr};
	if ($strMonth eq "Apr") {return "$strYear$yStr"."4$mStr$strDay".$dStr};
	if ($strMonth eq "May") {return "$strYear$yStr"."5$mStr$strDay".$dStr};
	if ($strMonth eq "Jun") {return "$strYear$yStr"."6$mStr$strDay".$dStr};
	if ($strMonth eq "Jul") {return "$strYear$yStr"."7$mStr$strDay".$dStr};
	if ($strMonth eq "Aug") {return "$strYear$yStr"."8$mStr$strDay".$dStr};
	if ($strMonth eq "Sep") {return "$strYear$yStr"."9$mStr$strDay".$dStr};	if ($strMonth eq "Oct") {return "$strYear$yStr"."10$mStr$strDay".$dStr};
	if ($strMonth eq "Nov") {return "$strYear$yStr"."11$mStr$strDay".$dStr};
	if ($strMonth eq "Dec") {return "$strYear$yStr"."12$mStr$strDay".$dStr};
	return "$week, $strDay $strMonth $strYear";
}